"use strict";

/* global response */

/* Script Includes */
var DataDomeAPI = require("*/cartridge/scripts/api/dataDomeAPI");

/* API Includes */
var Status = require("dw/system/Status");
var URLUtils = require("dw/web/URLUtils");

/**
 * This method will split the html of the CAPTCHA sent by the API server to get the request URL and the html without this request URL.
 * We split the html and the URL to store them in two different values of session.privacy to not trigger a session quota warning or error
* @param {string} html - the html of the CAPTCHA
* @returns {Object} {requestURL, responseHTMLWithoutURL}
 */
function splitHTML(html) {
    var requestURL = "";
    var responseHTMLWithoutURL = "";
    if (html && html.match(/'requestUrl'\s*:\s*'(.*)'/)) {
        requestURL = html.match(/'requestUrl'\s*:\s*'(.*)'/)[1]; // API server already truncates it to 1200
        // replace the part of the html that matches the request URL
        responseHTMLWithoutURL = html.replace(requestURL, "");
    }
    return {
        requestURL: requestURL,
        responseHTMLWithoutURL: responseHTMLWithoutURL
    };
}

/**
 * This method will intercept all incoming requests in the storefront and will only validate requests
 * that meets certain criteria via DataDomeAPI. The method will skip remote includes requests.
 * If the request has to be validated then will invoke DataDomeAPI to perform validation. It will redirect
 * incoming requests if DataDome Protection API decides to block invalid requests.
 * @returns {Object} ddResponse
 */
function validate() {
    if (!request.includeRequest) {
        var ddLogger = require("*/cartridge/scripts/util/ddLogger.js");
        try {
            var ddResponse = DataDomeAPI.validate();

            if (ddResponse.invalid) {
                var redirectUrl;
                if (ddResponse.redirectExternal) {
                    redirectUrl = ddResponse.location;
                } else {
                    var responseHTML = ddResponse.errorHtml;

                    var maxLengthWarningThreshold = 1200;

                    if (responseHTML.length >= maxLengthWarningThreshold) {
                        ddLogger.log("HTML payload for the CAPTCHA is longer than " + maxLengthWarningThreshold + " characters", "error", "DataDomeRequestFilter~validate");
                        var splitCaptcha = splitHTML(responseHTML);
                        var requestURL = splitCaptcha.requestURL;
                        var responseHTMLWithoutURL = splitCaptcha.responseHTMLWithoutURL;
                        session.privacy.ddRequestURL = requestURL;
                        session.privacy.ddResponseHTMLWithoutURL = responseHTMLWithoutURL;
                    } else {
                        session.privacy.ddResponseHTML = responseHTML;
                    }

                    var requestPath = request.getHttpPath();
                    var requestQueryString = request.getHttpQueryString();
                    redirectUrl = URLUtils.https("DDUser-Challenge").toString() + "?redirect=" + encodeURIComponent(requestPath + (requestQueryString ? "?" + requestQueryString : ""));
                }

                response.redirect(redirectUrl);
            }
        } catch (e) {
            ddLogger.log("An error occured while trying to parse DataDome response" + e, "error", "DataDomeRequestFilter~validate");
        }
    }

    return new Status(Status.OK);
}

/* Module Exports */
module.exports = {
    onRequest: validate
};
