"use strict";

/* global dw */

/* API Includes */
var Calendar = require("dw/util/Calendar");
var Site = require("dw/system/Site");
var HashMap = require("dw/util/HashMap");
var mapHelper = require("*/cartridge/scripts/util/mapHelper");

/**
 * This method is used to check the size of provided request parameter, if it's below or equal to limit then
 * It returns the original value but if it's more than limit then it will truncate the value and return that
 * truncated value.
 * @param {string} requestParam - request parameter
 * @Param {number} limit - request param size
 * @returns {string} requestParameter - truncated request parameter
 */
function truncateRequestParam(requestParam, limit) {
    if (requestParam == null) {
        return "";
    }

    return limit < 0 ? requestParam.slice(limit) : requestParam.slice(0, limit);
}

/**
 * This method is used to build request params by providing their limit in bytes, using RequestHandler method to truncate
 * params value if they exceed the provided limit. These limits are encouraged by Data Dome API.
 * @param {Request} request - current request object
 * @returns {HashMap} requestParamsMap - request params in the form of key value pairs
 */
function buildRequestParams(request) {
    var clientId;
    var siteCustomPreferences = Site.getCurrent().getPreferences().custom;
    var requestHeaders = request.getHttpHeaders();
    var requestHeadersCaseInsensitive = new mapHelper.CaseInsensitiveMap(requestHeaders);
    var requestHeaderNamesCommaSeparated = (!requestHeaders.isEmpty()) ? requestHeaders.keySet().iterator().asList().join() : "";
    var requestParamsMap = new HashMap();
    var originalRequestPath = request.getHttpPath();
    var originalRequestQueryParams = request.getHttpQueryString();
    var originalRequestUrl = originalRequestQueryParams ? originalRequestPath + "?" + originalRequestQueryParams : originalRequestPath;
    var timeRequestMilliSeconds = (new Calendar()).getTime().getTime();
    var timeRequestMicroSeconds = timeRequestMilliSeconds * 1000;

    requestParamsMap.put("Key", siteCustomPreferences.ddApiKey);
    requestParamsMap.put("RequestModuleName", "SFCC");
    requestParamsMap.put("ModuleVersion", "20.6.0");
    requestParamsMap.put("IP", request.getHttpRemoteAddress());
    requestParamsMap.put("TimeRequest", timeRequestMicroSeconds);
    requestParamsMap.put("Protocol", request.getHttpProtocol());
    requestParamsMap.put("Method", request.getHttpMethod());
    requestParamsMap.put("UserAgent", truncateRequestParam(request.getHttpUserAgent(), 768));
    requestParamsMap.put("ServerHostname", truncateRequestParam(request.getHttpHost(), 512));
    requestParamsMap.put("ServerName", truncateRequestParam(dw.system.Site.getCurrent().getHttpHostName(), 512));
    requestParamsMap.put("Host", truncateRequestParam(request.getHttpHost(), 512));
    requestParamsMap.put("HeadersList", truncateRequestParam(requestHeaderNamesCommaSeparated, 512));
    requestParamsMap.put("Port", 0);
    requestParamsMap.put("Referer", truncateRequestParam(request.getHttpReferer(), 1024));
    requestParamsMap.put("Request", truncateRequestParam(originalRequestUrl, 2048));

    var cookiesInfo = getClientIdAndCookiesLength(request);
    // session by header
    if (requestHeadersCaseInsensitive.containsKey('x-datadome-clientid')) {
        clientId = requestHeadersCaseInsensitive.get('x-datadome-clientid');
        requestParamsMap.put("X-DataDome-X-Set-Cookie", "true");
    } else {
        clientId = cookiesInfo.clientId;
    }
    requestParamsMap.put("ClientID", truncateRequestParam(clientId, 128));
    requestParamsMap.put("CookiesLen", cookiesInfo.cookiesLength);

    if (session.customer && !session.customer.anonymous) {
        requestParamsMap.put("UserId", session.customer.ID);
    }

    if (requestHeadersCaseInsensitive.containsKey("Accept-Encoding")) {
        requestParamsMap.put("AcceptEncoding", truncateRequestParam(requestHeadersCaseInsensitive.get("Accept-Encoding"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Accept-Language")) {
        requestParamsMap.put("AcceptLanguage", truncateRequestParam(requestHeadersCaseInsensitive.get("Accept-Language"), 256));
    }
    if (requestHeadersCaseInsensitive.containsKey("Accept")) {
        requestParamsMap.put("Accept", truncateRequestParam(requestHeadersCaseInsensitive.get("Accept"), 512));
    }
    if (requestHeadersCaseInsensitive.containsKey("Accept-Charset")) {
        requestParamsMap.put("AcceptCharset", truncateRequestParam(requestHeadersCaseInsensitive.get("Accept-Charset"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Content-Length")) {
        requestParamsMap.put("PostParamLen", requestHeadersCaseInsensitive.get("Content-Length"));
    }
    if (requestHeadersCaseInsensitive.containsKey("Content-Type")) {
        requestParamsMap.put("ContentType", truncateRequestParam(requestHeadersCaseInsensitive.get("Content-Type"), 64));
    }
    if (requestHeadersCaseInsensitive.get("Authorization")) {
        requestParamsMap.put("AuthorizationLen", requestHeadersCaseInsensitive.get("Authorization").length);
    } 
    if (requestHeadersCaseInsensitive.containsKey("Origin")) {
        requestParamsMap.put("Origin", truncateRequestParam(requestHeadersCaseInsensitive.get("Origin"), 512));
    }
    if (requestHeadersCaseInsensitive.containsKey("X-Forwarded-For")) {
        requestParamsMap.put("XForwardedForIP", truncateRequestParam(requestHeadersCaseInsensitive.get("X-Forwarded-For"), -512));
    }
    if (requestHeadersCaseInsensitive.containsKey("X-Requested-With")) {
        requestParamsMap.put("X-Requested-With", truncateRequestParam(requestHeadersCaseInsensitive.get("X-Requested-With"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Connection")) {
        requestParamsMap.put("Connection", truncateRequestParam(requestHeadersCaseInsensitive.get("Connection"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Pragma")) {
        requestParamsMap.put("Pragma", truncateRequestParam(requestHeadersCaseInsensitive.get("Pragma"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Cache-Control")) {
        requestParamsMap.put("CacheControl", truncateRequestParam(requestHeadersCaseInsensitive.get("Cache-Control"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("From")) {
        requestParamsMap.put("From", truncateRequestParam(requestHeadersCaseInsensitive.get("From"), 128));
    } 
    if (requestHeadersCaseInsensitive.containsKey("X-Real-IP")) {
        requestParamsMap.put("X-Real-IP", truncateRequestParam(requestHeadersCaseInsensitive.get("X-Real-IP"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Via")) {
        requestParamsMap.put("Via", truncateRequestParam(requestHeadersCaseInsensitive.get("Via"), 256));
    }
    if (requestHeadersCaseInsensitive.containsKey("True-Client-IP")) {
        requestParamsMap.put("TrueClientIP", truncateRequestParam(requestHeadersCaseInsensitive.get("True-Client-IP"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-UA-Mobile")) {
        requestParamsMap.put("SecCHUAMobile", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-UA-Mobile"), 8));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-Device-Memory")) {
        requestParamsMap.put("SecChDeviceMemory", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-Device-Memory"), 8));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-UA-Arch")) {
        requestParamsMap.put("SecCHUAArch", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-UA-Arch"), 16));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-UA-Platform")) {
        requestParamsMap.put("SecCHUAPlatform", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-UA-Platform"), 32));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-UA")) {
        requestParamsMap.put("SecCHUA", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-UA"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-UA-Model")) {
        requestParamsMap.put("SecCHUAModel", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-UA-Model"), 128));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-CH-UA-Full-Version-List")) {
        requestParamsMap.put("SecCHUAFullVersionList", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-CH-UA-Full-Version-List"), 256));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-Fetch-User")) {
        requestParamsMap.put("SecFetchUser", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-Fetch-User"), 8));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-Fetch-Dest")) {
        requestParamsMap.put("SecFetchDest", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-Fetch-Dest"), 32));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-Fetch-Mode")) {
        requestParamsMap.put("SecFetchMode", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-Fetch-Mode"), 32));
    }
    if (requestHeadersCaseInsensitive.containsKey("Sec-Fetch-Site")) {
        requestParamsMap.put("SecFetchSite", truncateRequestParam(requestHeadersCaseInsensitive.get("Sec-Fetch-Site"), 64));
    }

    return requestParamsMap;
}

function getClientIdAndCookiesLength(request) {

    var httpRequestCookies = request.getHttpCookies();
    var clientId = null;
    var cookiesLength = 0;


    for (var i = 0; i < httpRequestCookies.getCookieCount(); i++) {
        var cookie = httpRequestCookies[i];
        cookiesLength += cookie.getValue().length;
        if (cookie.getName() === "datadome") {
            clientId = cookie.getValue();
        }
    }

    return {
        clientId: clientId,
        cookiesLength: cookiesLength,
    };

}


/* Module Exports */
exports.buildRequestParams = buildRequestParams;
